import logging

from rocolib.api.components import Component
from rocolib.api.ports import AnchorPort
from rocolib.utils.transforms import np, Transform6DOF


log = logging.getLogger(__name__)

def vals(x):
    if x is None:
        return None
    return list(map(lambda p: p.getValue(), x))

class MechanicalComponent(Component):
    def predefine(self, **kwargs):
        self._origin = [ self.addParameter("_d"+x, 0, paramType="length", minValue=None, hidden=True)
            for x in "xyz"
        ]

        if kwargs.get("euler", False):
            self._euler = [ self.addParameter(x, 0, paramType="angle", hidden=True)
                for x in "_roll _pitch _yaw".split()
            ]
        else:
            self._euler = None
            self._quat = [ self.addParameter("_q_"+x, int(x == "a"),
                                valueType="(int, float)", minValue=-1, maxValue=1, hidden=True)
                for x in "aijk"
            ]
            #self.addSemanticConstraint(np.Eq(np.norm(self._quat), 1))

        #self.addInterface("", TPort())

    def get6DOF(self):
        return Transform6DOF(vals(self._origin), vals(self._euler), vals(self._quat))

    def makeOutput(self, *args, **kwargs):
        ## XXX Duplicated from component to set parameters for transform
        if kwargs.get("remake", True):
            self.make(kwargs.get("useDefaultParameters", True))
            log.debug(f"... done making {self.getName()}.")
            kwargs["remake"] = False

        if "transform3D" not in kwargs:
            kwargs["transform3D"] = self.get6DOF()

        return Component.makeOutput(self, *args, **kwargs)
